package bip.gwm.scm.settlement.utils;

import bip.gwm.scm.settlement.utils.BusinessUtils;
import bip.gwm.synchronize.SyncInvoicePush;
import cn.hutool.core.codec.Base64Decoder;
import cn.hutool.core.util.HexUtil;
import cn.hutool.crypto.BCUtil;
import cn.hutool.crypto.SmUtil;
import cn.hutool.crypto.asymmetric.SM2;
import cn.hutool.http.ContentType;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import kd.bos.dataentity.entity.DynamicObject;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

@Service
public class OpenApiDemoSm2Utils {

   private static final Logger log = LoggerFactory.getLogger(SyncInvoicePush.class);
   private static final String SM2_PRI_KEY = "6DE16931F99B7C7905D247C973F9E203F1E6543D63E3CD7E13246F0D39A3CD8F";
   private static final String SM2_PUB_KEY = "04107d798a9ee73b109dcee80ef95b62143ffcd4258a500e7b29600003035911c007357db3034172bbcfa3616c4f4658352b77f50b49c89cb772d09baa4b18202e";
   private static final String SM2_CMB_PUB_KEY = "04e90f9f92db2763d3853fe2e9491e5475bc5fe731c214ed0f98e2a514d4f10c81a5f23b0f6db07ff444f6dcd57e69c4b3e05124cc3ef8b16da288d54744b88a1e";
   private static final String APP_ID = "c1defa9b-e1c1-42a1-b4a6-4ecec372f1a4";
   private static final String SECRET = "9a014725-859e-4ed1-ae54-c360d549ff96";
   private static final String VERIFY = "SM3withSM2";
   private static final String confirmRightsURL = "https://api.cmburl.cn:8065/skt/exima/ver1/jyt-settle";
   private static final String DUIZHANGURL = "https://api.cmburl.cn:8065/skt/exima/rec_center/downloadUrl";


   public static SM2 SM2_INSTANCE(DynamicObject platformMerchants) {
      return SmUtil.sm2(platformMerchants.getString("bip_merchant_private_k"), platformMerchants.getString("bip_public_key"));
   }

   public JSONObject confirmRightsRequest(JSONObject reqVo, String URL, DynamicObject platformMerchants) {
      String resp = this.httpClientPost(this.buildHeader(sm3DigestHex(reqVo.toJSONString()), platformMerchants), reqVo.toJSONString(), URL, platformMerchants);
      return JSON.parseObject(resp);
   }

   public String sm2SignHex(String dataStr, DynamicObject dynamicObject) {
      byte[] sign = SM2_INSTANCE(dynamicObject).sign(dataStr.getBytes(StandardCharsets.UTF_8));
      byte[] signature = SmUtil.rsAsn1ToPlain(sign);
      return HexUtil.encodeHexStr(signature);
   }

   public static boolean sm2Verify(String msgStr, String signatureStr, DynamicObject dynamicObject) {
      return SM2_INSTANCE(dynamicObject).verify(msgStr.getBytes(StandardCharsets.UTF_8), SmUtil.rsPlainToAsn1(SmUtil.rsAsn1ToPlain(Base64Decoder.decode(signatureStr))));
   }

   public static String sm3DigestHex(String src) {
      try {
         return SmUtil.sm3().digestHex(src);
      } catch (Exception var2) {
         throw new RuntimeException("签名计算出现异常");
      }
   }

   public String httpClientPost(Map header, String body, String url, DynamicObject platformMerchants) {
      log.info("HttpVisitor body is {},header is {},url is {}", new Object[]{body, header, url});

      try {
         HttpResponse e = ((HttpRequest)HttpRequest.post(url).body(body, ContentType.JSON.toString()).header(header)).timeout(10000).setReadTimeout(30000).execute();
         if(200 != e.getStatus()) {
            log.error("请求失败：status is [{}],message is [{}]", Integer.valueOf(e.getStatus()), e.body());
         }

         String cmbBodySign = (String)((List)e.headers().get("CMB-BodySign")).get(0);
         if(!StringUtils.isEmpty(cmbBodySign)) {
            boolean respBody = sm2Verify(e.body(), cmbBodySign, platformMerchants);
            if(!respBody) {
               log.error("验签失败");
            } else {
               log.info("验签成功");
            }
         }

         String respBody1 = e.body();
         log.info("HttpVisitor resp = {}", respBody1);
         return respBody1;
      } catch (Exception var8) {
         log.error("Error: http post error: body is {},header is {},url is {}, e = ", new Object[]{body, header, url, var8});
         throw var8;
      }
   }

   public static void generateSm2KeyPair() {
      SM2 sm2 = SmUtil.sm2();
      byte[] privateKey = BCUtil.encodeECPrivateKey(sm2.getPrivateKey());
      byte[] publicKey = ((BCECPublicKey)sm2.getPublicKey()).getQ().getEncoded(false);
      System.out.println("私钥: " + HexUtil.encodeHexStr(privateKey).toUpperCase());
      System.out.println("公钥: " + HexUtil.encodeHexStr(publicKey).toUpperCase());
   }

   private Map buildHeader(String sign, DynamicObject platformMerchants) {
      String timestamp = String.valueOf(System.currentTimeMillis() / 1000L);
      HashMap headerNeedMd5Data = new HashMap(4);
      headerNeedMd5Data.put("appid", platformMerchants.getString("bip_appid"));
      headerNeedMd5Data.put("secret", platformMerchants.getString("bip_app_secret"));
      headerNeedMd5Data.put("sign", sign);
      headerNeedMd5Data.put("timestamp", timestamp);
      String headerNeedMd5SortStr = this.buildSortData(headerNeedMd5Data);
      HashMap header = new HashMap(4);
      header.put("appid", Collections.singletonList(platformMerchants.getString("bip_appid")));
      header.put("timestamp", Collections.singletonList(timestamp));
      header.put("apisign", Collections.singletonList(this.sm2SignHex(headerNeedMd5SortStr, platformMerchants)));
      header.put("sign", Collections.singletonList(sign));
      header.put("verify", Collections.singletonList("SM3withSM2"));
      return header;
   }

   private String buildSortData(Map map) {
      ArrayList keys = new ArrayList(map.keySet());
      StringBuilder content = new StringBuilder();
      Collections.sort(keys);
      int index = 0;
      Iterator var5 = keys.iterator();

      while(var5.hasNext()) {
         String key = (String)var5.next();
         String value = (String)map.get(key);
         if(null != key && null != value) {
            content.append(index == 0?"":"&").append(key).append("=").append(value);
            ++index;
         }
      }

      return content.toString();
   }

   public static void main(String[] args) {
      try {
         new BigDecimal("123.21");
         generateSm2KeyPair();
         JSONObject reqVo = new JSONObject();
         reqVo.put("merchId", "3089991527105OB");
         reqVo.put("clearDate", "20240827");
         String confirmRightsURL = "https://api.cmburl.cn:8065/skt/exima/rec_center/downloadUrl";
         OpenApiDemoSm2Utils openApiDemoService = new OpenApiDemoSm2Utils();
         DynamicObject platformMerchants = BusinessUtils.getPlatformMerchants(Long.valueOf(2028462395273249792L));
         openApiDemoService.confirmRightsRequest(reqVo, confirmRightsURL, platformMerchants);
         System.out.println("result is " + openApiDemoService.confirmRightsRequest(reqVo, confirmRightsURL, platformMerchants));
      } catch (Exception var7) {
         var7.printStackTrace();
      }

   }

}
